@@ -1,16 +1,7 @@
-# Copyright 2014 Google Inc.
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# Copyright (c) 2014-2016 Google, Inc.
+# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+
 """Check Python 2 code for Python 2/3 source-compatible issues."""
 from __future__ import absolute_import, print_function
 
@@ -19,7 +10,6 @@ import tokenize
 
 import astroid
 from astroid import bases
-from astroid import helpers
 
 from pylint import checkers, interfaces
 from pylint.utils import WarningScope
@@ -410,7 +400,6 @@ class Python3Checker(checkers.BaseChecker):
     def visit_print(self, node):
         self.add_message('print-statement', node=node)
 
-    @utils.check_messages('no-absolute-import', 'import-star-module-level')
     def visit_importfrom(self, node):
         if node.modname == '__future__':
             for name, _ in node.names:
@@ -419,10 +408,12 @@ class Python3Checker(checkers.BaseChecker):
                 elif name == 'absolute_import':
                     self._future_absolute_import = True
         elif not self._future_absolute_import:
-            self.add_message('no-absolute-import', node=node)
+            if self.linter.is_message_enabled('no-absolute-import'):
+                self.add_message('no-absolute-import', node=node)
         if node.names[0][0] == '*':
-            if not isinstance(node.scope(), astroid.Module):
-                self.add_message('import-star-module-level', node=node)
+            if self.linter.is_message_enabled('import-star-module-level'):
+                if not isinstance(node.scope(), astroid.Module):
+                    self.add_message('import-star-module-level', node=node)
 
     @utils.check_messages('no-absolute-import')
     def visit_import(self, node):
@@ -448,7 +439,7 @@ class Python3Checker(checkers.BaseChecker):
         kwargs = []
         if (isinstance(node.func, astroid.Attribute)
                 and node.func.attrname == 'sort'):
-            inferred = helpers.safe_infer(node.func.expr)
+            inferred = utils.safe_infer(node.func.expr)
             if not inferred:
                 return
 
@@ -459,7 +450,7 @@ class Python3Checker(checkers.BaseChecker):
 
         elif (isinstance(node.func, astroid.Name)
               and node.func.name == 'sorted'):
-            inferred = helpers.safe_infer(node.func)
+            inferred = utils.safe_infer(node.func)
             if not inferred:
                 return
 
