@@ -1,16 +1,8 @@
-# Copyright (c) 2003-2013 LOGILAB S.A. (Paris, FRANCE).
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# Copyright (c) 2003-2016 LOGILAB S.A. (Paris, FRANCE).
+# http://www.logilab.fr/ -- mailto:contact@logilab.fr
+# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+
 """utilities for Pylint configuration :
 
 * pylintrc
@@ -28,6 +20,7 @@ from __future__ import print_function
 import contextlib
 import collections
 import copy
+import io
 import optparse
 import os
 import pickle
@@ -35,7 +28,10 @@ import re
 import sys
 import time
 
-from six.moves import configparser
+try:
+    import configparser
+except ImportError:
+    from six.moves import configparser
 from six.moves import range
 
 from pylint import utils
@@ -161,6 +157,9 @@ def _regexp_validator(_, name, value):
         return value
     return re.compile(value)
 
+# pylint: disable=unused-argument
+def _regexp_csv_validator(_, name, value):
+    return [_regexp_validator(_, name, val) for val in _csv_validator(_, name, value)]
 
 def _yn_validator(opt, _, value):
     if isinstance(value, int):
@@ -177,6 +176,7 @@ VALIDATORS = {
     'string': utils._unquote,
     'int': int,
     'regexp': re.compile,
+    'regexp_csv': _regexp_csv_validator,
     'csv': _csv_validator,
     'yn': _yn_validator,
     'choice': lambda opt, name, value: _choice_validator(opt['choices'], name, value),
@@ -254,10 +254,14 @@ def _multiple_choices_validating_option(opt, name, value):
 
 
 class Option(optparse.Option):
-    TYPES = optparse.Option.TYPES + ('regexp', 'csv', 'yn', 'multiple_choice')
+    TYPES = optparse.Option.TYPES + ('regexp', 'regexp_csv', 'csv', 'yn',
+                                     'multiple_choice',
+                                     'non_empty_string')
+
     ATTRS = optparse.Option.ATTRS + ['hide', 'level']
     TYPE_CHECKER = copy.copy(optparse.Option.TYPE_CHECKER)
     TYPE_CHECKER['regexp'] = _regexp_validator
+    TYPE_CHECKER['regexp_csv'] = _regexp_csv_validator
     TYPE_CHECKER['csv'] = _csv_validator
     TYPE_CHECKER['yn'] = _yn_validator
     TYPE_CHECKER['multiple_choice'] = _multiple_choices_validating_option
@@ -616,7 +620,12 @@ class OptionsManagerMixIn(object):
             config_file = os.path.expanduser(config_file)
         if config_file and os.path.exists(config_file):
             parser = self.cfgfile_parser
-            parser.read([config_file])
+
+            # Use this encoding in order to strip the BOM marker, if any.
+            with io.open(config_file, 'r', encoding='utf_8_sig') as fp:
+                # pylint: disable=deprecated-method
+                parser.readfp(fp)
+
             # normalize sections'title
             for sect, values in list(parser._sections.items()):
                 if not sect.isupper() and values:
@@ -660,7 +669,7 @@ class OptionsManagerMixIn(object):
             else:
                 args = list(args)
             (options, args) = self.cmdline_parser.parse_args(args=args)
-            for provider in self._nocallback_options.keys():
+            for provider in self._nocallback_options:
                 config = provider.config
                 for attr in config.__dict__.keys():
                     value = getattr(options, attr, None)
