@@ -1,18 +1,8 @@
-# Copyright (c) 2000-2013 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2003-2016 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
-#
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+
 """handle diagram generation options for class diagram or default diagrams
 """
 
@@ -49,10 +39,7 @@ class DiaDefGenerator(object):
         # if we have a class diagram, we want more information by default;
         # so if the option is None, we return True
         if option is None:
-            if self.config.classes:
-                return True
-            else:
-                return False
+            return bool(self.config.classes)
         return option
 
     def _set_default_options(self):
@@ -216,9 +203,15 @@ class DiadefsHandler(object):
         self.config = config
 
     def get_diadefs(self, project, linker):
-        """get the diagrams configuration data
-        :param linker: pyreverse.inspector.Linker(IdGeneratorMixIn, LocalsVisitor)
-        :param project: pyreverse.utils.Project
+        """Get the diagrams configuration data
+
+        :param project:The pyreverse project
+        :type project: pyreverse.utils.Project
+        :param linker: The linker
+        :type linker: pyreverse.inspector.Linker(IdGeneratorMixIn, LocalsVisitor)
+
+        :returns: The list of diagram definitions
+        :rtype: list(:class:`pylint.pyreverse.diagrams.ClassDiagram`)
         """
 
         #  read and interpret diagram definitions (Diadefs)
